function [alphas_previous_iteration, betas_previous_iteration, mus_previous_iteration, sigmas_previous_iteration, nus_previous_iteration, lambdas_previous_iteration, total_model_log_like , iteration_finished , returnmessage ] = mix2( temp_fname, alphas, betas, mus, sigmas, nus, lambdas, update_alphas, update_betas, update_mus, update_sigmas, update_nus, update_lambdas, min_log_like_diff, min_par_diff, array_groupid, nr_transcripts, nr_blocks, nr_fragments, trans_lengths, nr_iterations, temp_fss, temp_fssv, temp_fl, temp_flv, pl_params_mu, pl_params_sigma , nus_pairing , lambdas_pairing )

	if isempty(nus_pairing)
		nus_pairing = [1:nr_transcripts];
	end
	if isempty(lambdas_pairing)
		lambdas_pairing = [1:nr_transcripts];
	end
	warning('off');
	warning_NaN = 0;	%There is will only be one warning for NaN states in the lambdas variable. If set to 1 there will be no warning at all.
	if nr_transcripts != length(alphas) error('numbers of transcripts and alphas have to be the same'); end	%This should not happen anyway.
	if nr_transcripts != length(array_groupid) error('numbers of transcripts and group ids have to be the same'); end	%Should not occur at all.
	nr_groups = max(array_groupid);
	% mat3 matrices dimensions are: fragments x blocks x transcripts
	mat3_fss = repmat( temp_fss ,[1 nr_blocks 1]); %mat3_fss varies in first and third dimension (=fragment + transcript dimension), constant in second dimension
	mat3_fl = repmat(temp_fl,[1 nr_blocks 1]);	%Dito
	mat3_fss_validity = repmat( temp_fssv , [ 1 nr_blocks 1]);
	mat3_fl_validity = repmat( temp_flv ,[1 nr_blocks 1]);	
	irs_index = [1:nr_fragments];
	iat_index = [1:nr_transcripts];
	ib_index = [1:nr_blocks];
	for iat=1:nr_transcripts
		index_norm( irs_index , iat ) =  trans_lengths(iat) + 1 - mat3_fss( irs_index , 1 , iat) ;
	end
	alphas_previous_iteration = alphas;
	mus_previous_iteration = mus; 
	sigmas_previous_iteration = sigmas;
	lambdas_previous_iteration = lambdas;
	nus_previous_iteration = nus;
	%init_groups
	for gid=1:nr_groups
		%This loop will assign the transcripts to cells of groups, and write index information.
		scalar_count=0;
		for iat = 1:nr_transcripts
			if(array_groupid(iat) == gid) iat_index_groups{gid}(++scalar_count) = iat;	 end 
		end
		nr_transcripts_groups{gid} = sum(array_groupid==gid);
		if length(iat_index_groups{gid}) != nr_transcripts_groups{gid}
			error('numbers of transcripts and iat_index length inside the groups have to be the same');
		end
		betas_previous_iteration{gid} = betas{gid};	
		nr_blocks_groups{gid} = nr_blocks;
		iteration_results_betas{gid}{1} = betas{gid};
	end
	iteration_results_alphas{1} = alphas;
	iteration_results_mus{1} = mus;
	iteration_results_sigmas{1} = sigmas;
	iteration_results_nus{1} = nus;
	iteration_results_lambdas{1} = lambdas;
	for iter = 1:nr_iterations
	% this is the expectation step; here we calculate the stats for the model that was updated in the previous iteration
		%Obligatory reset of the parameters at the begin of the "iterations" for loop. 
		alphas = alphas_previous_iteration;
		mus = mus_previous_iteration;
		sigmas = sigmas_previous_iteration;
		lambdas = lambdas_previous_iteration;
		nus = nus_previous_iteration;
		for gid=1:nr_groups;
			betas{gid} = betas_previous_iteration{gid};
		end
	%Here is the information of each parameter spread over 3 dimensions. This reduces the different EM calculations to a sequence of summations and divison operations of a single matrix. It also eased debugging and the analysis of different states during the EM algorithm.
		temp_al(1,1,:) = alphas;
		mat3_alphas = repmat(temp_al,[ nr_fragments , nr_blocks , 1 ]); %mat3_alphas varies only in last dimension (=transcript dimension), constant in first two dimensions
		temp_mu(1,:,1) = mus;
		mat3_mus = repmat(temp_mu,[nr_fragments 1 nr_transcripts]);
		temp_nu(1,1,:) = nus;
		mat3_nus = repmat(temp_nu , [nr_fragments , nr_blocks , 1] );
		temp_si(1,:,1) = sigmas;
		mat3_sigmas = repmat(temp_si,[nr_fragments 1 nr_transcripts]);
		temp_la(1,1,:) = lambdas;
		mat3_lambdas = repmat(temp_la , [nr_fragments , nr_blocks , 1] );
		for gid=1:nr_groups	
			temp_be{gid}(1,:,1) = betas{gid};
			mat3_betas{gid} = repmat(temp_be{gid},[ nr_fragments , 1 , nr_transcripts ]);
		end
		p_mus = (mat3_fss .- mat3_nus) ./ mat3_lambdas;
		p_sig = ((mat3_fss .- mat3_nus) ./ mat3_lambdas - mat3_mus ).^2;
		p_nus_a = (mat3_fss .- mat3_mus .* mat3_lambdas) ./ ((mat3_sigmas).^2);
		p_nus_b = (mat3_sigmas).^2;
		p_lambdas_a = (mat3_mus .* (mat3_fss .- mat3_nus)) ./ (mat3_sigmas.^2);
		p_lambdas_b = (mat3_fss .- mat3_nus).^2;
		p_lambdas_c = (mat3_sigmas.^2);
		%pdf for fragment lengths
		for iat=1:nr_transcripts
			temp_pl(:,1,iat) = normpdf( mat3_fl (:,1,iat) , pl_params_mu , pl_params_sigma );
			if(!update_mus && !update_sigmas && !sum(update_nus) && !sum(update_lambdas) ) % no need to work in continuous probability space, normalize to discrete probability space
				vector_normalizer = cumsum( normpdf( [1:trans_lengths(iat)]' , pl_params_mu , pl_params_sigma ) );
				temp_pl( irs_index , 1 , iat ) = temp_pl( irs_index , 1 , iat ) ./ vector_normalizer( index_norm( irs_index , iat) , 1 ) ;
			end
		end
		mat3_prob_length_given_trans = repmat(temp_pl,[1 nr_blocks 1]) .* mat3_fl_validity;		
		if( all(all(all( mat3_prob_length_given_trans == 0 ))) )	%If no fragment length values could be retrieved from the data
			default_value = 1;		
		else
			default_value = min(min(min( mat3_prob_length_given_trans ( mat3_prob_length_given_trans > 0) )));
		end
		mat3_prob_length_given_trans ( mat3_prob_length_given_trans == 0) = default_value ;
		%Group-wise
		for gid = 1:nr_groups
			% pdf for fragment startsites
			for iat = 1:nr_transcripts
				for ibt = 1:nr_blocks
					mat3_prob_startsite_given_block_trans_group{gid}(:,ibt,iat) = normpdf( mat3_fss(:,ibt,iat) , lambdas(iat)*mus(ibt)+nus(iat), sigmas(ibt)*lambdas(iat) );
					if(!update_mus && !update_sigmas && !sum(update_nus) && !sum(update_lambdas))
						discrete_norm = sum( normpdf([1:trans_lengths(iat)] , lambdas(iat)*mus(ibt) + nus(iat) , sigmas(ibt) * lambdas(iat)) );
						mat3_prob_startsite_given_block_trans_group{gid}(:,ibt,iat) /= discrete_norm;
					end
				end
			end
			mat3_prob_given_block_trans_group{gid} = mat3_prob_startsite_given_block_trans_group{gid} .* mat3_alphas .* mat3_betas{gid} .* mat3_fss_validity .* mat3_prob_length_given_trans ;
			prob_frag_trans_group{gid} = sum(mat3_prob_given_block_trans_group{gid}(:,:,iat_index_groups{gid}),2); %sum over blocks for the group, sum_j_(g(j)=k)(ai*bj*p(r|t=i)
			prob_frag_block_group{gid} = sum(mat3_prob_given_block_trans_group{gid}(:,:,iat_index_groups{gid}),3); %sum over transcripts in group k, sum_i_(g(i)=k)(ai*bj*p(r|t=i)
			prob_frag_group{gid} = sum(sum(mat3_prob_given_block_trans_group{gid}(:,:,iat_index_groups{gid}),2),3); %sum over blocks and transcripts
			prob_block_given_frag_group{gid} = prob_frag_block_group{gid} ./ repmat( prob_frag_group{gid}, [1 nr_blocks 1] );
			prob_block_given_frag_group{gid}(isnan(prob_block_given_frag_group{gid})) = 0;
		end
		prob_frag_trans  = zeros(nr_fragments,1,nr_transcripts);
		prob_frag = zeros(nr_fragments,1);
		prob_frag_block_trans = zeros(nr_fragments, nr_blocks, nr_transcripts);
	%Group-wise
		for gid = 1:nr_groups
			 prob_frag  += prob_frag_group{gid};	%Sum over the groups
			 prob_frag_trans (:,1,iat_index_groups{gid}) +=  prob_frag_trans_group {gid};	% For calculation of alpha values that use the pdf values of all groups
		end
		temp_prob_frag = prob_frag ;
		temp_prob_frag (temp_prob_frag==0) = [];
		total_model_log_like = sum(log(temp_prob_frag)); % expectation step: this is the likelihood of the model that was updated in the previous iteration
		iteration_results_total_model_log_like(iter) = total_model_log_like;
		if( iter > 10 && (abs(iteration_results_total_model_log_like(iter-1) - iteration_results_total_model_log_like(iter)) < min_log_like_diff) )
			returnmessage = sprintf("Terminated the EM algorithm because difference in sum(log(likelihood)) between this iteration and the previous iteration was below %f.", min_log_like_diff);
			iteration_finished = iter;
			logdiff = abs(iteration_results_total_model_log_like(iter-1) - iteration_results_total_model_log_like(iter));
			break;
		endif
	%Group-wise
		for gid = 1:nr_groups
			prob_group_given_frag{gid} = prob_frag_group{gid} ./ prob_frag; % p(g=k|r)
			prob_frag_block_trans(:,:,iat_index_groups{gid}) = mat3_prob_given_block_trans_group{gid}(:,:,iat_index_groups{gid});	%The pdf values of prob_frag_block_trans_group for each group, selecting the pdf values associated with the transcripts of that group.
		end
		prob_trans_block_given_frag = prob_frag_block_trans ./ repmat( prob_frag, [1 nr_blocks nr_transcripts] );	%Unbiased selection of pdf values of the posterior probability prob_frag_block_trans_group.
		prob_trans_block_given_frag(isnan(prob_trans_block_given_frag)) = 0;
	% this is the maximization step; here we update the parameters
		%ALPHAS
		prob_trans_given_frag  =  prob_frag_trans  ./ repmat( prob_frag, [1 1 nr_transcripts] ); %p(t=i|r)
		prob_trans_given_frag (isnan(prob_trans_given_frag )) = 0;
		nr_fragments_non_zero_total_prob = sum(sign(prob_frag)); %total number of fragments which have probability > 0 for at least one transcript
		alphas_new(iat_index) = sum(prob_trans_given_frag ,1) / nr_fragments_non_zero_total_prob;	% 1/(reR) * sum(p(t=i|r),r)
		%MUS
		mus_new(ib_index) = sum(sum( prob_trans_block_given_frag .* p_mus ,1),3) ./ sum(sum( prob_trans_block_given_frag ,1),3);
		%SIGMAS
		sigmas_new(ib_index) = sqrt( sum(sum(prob_trans_block_given_frag .* p_sig ,1),3) ./ sum(sum(prob_trans_block_given_frag ,1),3) );
		%NUS
		nus_new_a= sum( sum( prob_trans_block_given_frag .* p_nus_a , 1 ) ,2 );
		nus_new_b = sum( sum( prob_trans_block_given_frag ./ p_nus_b ,1 ) ,2 );
		nus_new = reshape( nus_new_a ./ nus_new_b , 1 , nr_transcripts );
		nus_new(isnan(nus_new)) = nus(isnan(nus_new));
		% Nus-pairing
		for ignp =1:max(nus_pairing)
			index_pot = [1:nr_transcripts](nus_pairing == ignp);
			nus_new( index_pot ) = sum ( nus_new (index_pot ) ) / sum ( nus_pairing == ignp );
		end
		%LAMBDAS
		lambdas_a= sum(sum( prob_trans_block_given_frag ,1),2);
		lambdas_b = sum(sum( prob_trans_block_given_frag .* p_lambdas_a ,1),2);
		lambdas_c = -1 * sum(sum( prob_trans_block_given_frag .* p_lambdas_b ./ p_lambdas_c ,1),2);
		for iat=1:nr_transcripts
			lambdas_new(iat) = (-lambdas_b(:,:,iat)+sqrt(lambdas_b(:,:,iat)^2-4*lambdas_a(:,:,iat)*lambdas_c(:,:,iat)))/(2*lambdas_a(:,:,iat));
			lambdas_2ndresult(iat) = (-lambdas_b(:,:,iat)-sqrt(lambdas_b(:,:,iat)^2-4*lambdas_a(:,:,iat)*lambdas_c(:,:,iat)))/(2*lambdas_a(:,:,iat));
			if(isnan(lambdas_new(iat)) & warning_NaN==0 )
				printf("NaN warning: At least one solution of quadratic equation %ex^2+%ex+%e was NaN!\n",lambdas_a(:,:,iat),lambdas_b(:,:,iat),lambdas_c(:,:,iat));
				warning_NaN=1;
				fflush(stdout);
				lambdas_new(iat) = lambdas(iat);
			end
		endfor
		lambdas_new (isnan(lambdas_new)) = lambdas(isnan(lambdas_new));
		% Lambdas-pairing
		for ignp =1:max(lambdas_pairing)
			index_pot = [1:nr_transcripts](lambdas_pairing == ignp);
			lambdas_new( index_pot ) = sum ( lambdas_new (index_pot ) ) / sum ( lambdas_pairing == ignp );
		end
		%Group-wise
		for gid = 1:nr_groups
			%BETAS
			betas_new{gid}(ib_index) = sum( repmat(prob_group_given_frag{gid}, 1 , nr_blocks) .* prob_block_given_frag_group{gid} , 1)  ./ sum( repmat(prob_group_given_frag{gid}, 1 , nr_blocks) , 1);
			if(isnan(betas_new{gid}))
				betas_new{gid}=betas{gid};
			endif
			betas_diff(gid,:) = abs( betas{gid} .- betas_new{gid} );
		end
		alphas_diff(iat_index) = abs( alphas .- alphas_new );
		if(update_alphas)
			alphas_previous_iteration = alphas_new;
		end
		if(update_betas) 
			for gid=1:nr_groups
				betas_previous_iteration{gid} = betas_new{gid};
			end
		end
		mus_diff(ib_index) = abs( mus .- mus_new );
		if(update_mus)
			mus_previous_iteration = mus_new;
		end
		sigmas_diff(ib_index) = abs( sigmas .- sigmas_new );
		if(update_sigmas)
			sigmas_previous_iteration = sigmas_new;
		end
		nus_diff(iat_index) = abs( nus .- nus_new );
		if(sum(update_nus))
			nus_previous_iteration(update_nus==1) = nus_new(update_nus==1);
		end
		lambdas_diff(iat_index) = abs( lambdas .- lambdas_new );
		if(sum(update_lambdas))
			lambdas_previous_iteration(update_lambdas==1) = lambdas_new(update_lambdas==1);
		end
		if (sum(isnan(alphas_new)) > 0)
			returnmessage = sprintf("At least one element of the alpha parameter turned NaN during the estimation.");
			iteration_finished=iter;
			break;
		end
		if( (max(max( betas_diff )) < min_par_diff ) && (max( alphas_diff ) < min_par_diff ) && (max( mus_diff ) < min_par_diff ) && (max( sigmas_diff ) < min_par_diff ) && (max( nus_diff ) < min_par_diff ) && (max( lambdas_diff ) < min_par_diff ) )
			returnmessage = sprintf("Terminated the EM algorithm because the maximal difference of all parameter values between two iterations was below %f.", min_par_diff);
			iteration_finished=iter;
			break;
		endif
		for gid=1:nr_groups
			iteration_results_betas{gid}{iter+1} = betas_previous_iteration{gid};
		end	
		iteration_results_alphas{iter+1} = alphas_previous_iteration;
		iteration_results_mus{iter+1} = mus_previous_iteration;
		iteration_results_sigmas{iter+1} = sigmas_previous_iteration;
		iteration_results_nus{iter+1} = nus_previous_iteration;
		iteration_results_lambdas{iter+1} = lambdas_previous_iteration;
	end
	if iter==nr_iterations
		returnmessage = sprintf("Terminated the EM algorithm because the maximal number of iterations was reached.");
		iteration_finished=iter;
	end
	if temp_fname
		save("-v7",temp_fname,"iteration_results_alphas","iteration_results_betas","iteration_results_mus","iteration_results_sigmas","iteration_results_lambdas","iteration_results_nus","iteration_results_total_model_log_like");
	end
endfunction
