function  [ fragment_startsites_trans_coord , fragment_startsites_validity_mat , fragment_lengths_trans_coord , fragment_lengths_validity_mat , nr_reads_locus ] = validity_starts_lengths( trans_defs_file , samplefile , annotation_extension , gene_selection )
	if nargin <3
		annotation_extension=0;
	end
	if nargin <4
		gene_selection=0;
	end
	%Internal transcripts/gene definition from .gff TYPE 3 file
	if !iscell(trans_defs_file)
		[fdir,fname,fend] = fileparts(trans_defs_file);	%It is possible to provide gene annotations either as .gff type 3 or .gtf
		if strcmp(fend,".gff")
			[ gene ] = extract_transdefs_gff( trans_defs_file , gene_selection , 0 );
		elseif strcmp(fend,".gtf")
			[ gene ] = extract_transdefs_gtf( trans_defs_file , gene_selection , 0 );
		else
			error("did not recognize the file type of the gene/transcript annotation\n");
		end
		gene_id = gene{1};
		nr_transcripts = gene{2};
		nr_exons = gene{3};	
		trans_lengths = gene{4};
		region = gene{5};
		for iat=1:nr_transcripts
			trans_identity{iat} = gene{6}{iat}{1};
			trans_start(iat) = gene{end}{iat}{end}(1);
		end
	else
		gene_id = gene{1};
		nr_transcripts = gene{2};
		nr_exons = gene{3};	
		trans_lengths = gene{4};
		region = gene{5};
	end
	%filename differentiation
	[fdir,fname,fend] = fileparts(samplefile);
	if strcmp(fend,".sam")
		printf("\tInput was recognized as .sam file\n")
		fflush(stdout);
		[ reads_single, single_nr_exoncoords , fragment_ids_single, reads_paired , paired_nr_exoncoords , fragment_ids_paired , totalcount ] = util_extract_reads( samplefile , nr_exons*2 );
        size(single_nr_exoncoords)
        size(paired_nr_exoncoords{1})
        size(paired_nr_exoncoords{2})
        size(reads_single)
        reads_single(1,:)
	else
		error("Data file ending unrecognized. Can't deal with %s\n",fend);
	end
	nr_reads_single = size( fragment_ids_single , 2);
	nr_reads_paired = size( fragment_ids_paired , 2);
#	fragment_ids = [ fragment_ids_paired ; fragment_ids_single ];
	fragment_ids = [ fragment_ids_paired' ; fragment_ids_single' ];
	%Fragment startsites in transcript space and fragment lengths in transcript space are decoupled. These two matrices have identical dimensions ( 1st dimension: fragments, 2nd dimension : transcripts ), but have the different sizes of valid content and therefore have separate validity matrices. Single end reads generate only fragment_startsites_trans_coord values, while paired end reads generate fragment_startsites_trans_coord and fragment_lengths_trans_coord values. Depending on the type of the reads contained in the samplefile there may be more startsite values than length values. A sample file that only contains single ends reads generates no fragment length values at all.
	fragment_lengths_trans_coord = zeros( 1 , nr_transcripts); 
	fragment_startsites_trans_coord = zeros( 1 , nr_transcripts);	
	fragment_validity_mat = zeros( 1 ,nr_transcripts);
	fragment_lengths_trans_coord_paired = [];
	fragment_lengths_trans_coord_single = [];
	fragment_lengths_validity_mat_paired = [];
	fragment_lengths_validity_mat_single = []; 
	fragment_startsites_validity_mat_paired = []; 
	fragment_startsites_validity_mat_single = [];
	fragment_startsites_trans_coord_paired = [];
	fragment_startsites_trans_coord_single = [];
	fragment_ids_startsites = [];
	fragment_ids_lengths = [];
	read_lengths_single_trans_coord = zeros( 1 ,nr_transcripts);
	% Creating vectorized index matrices to access the read matrices
	[ rf_r , rf_c ] = size( reads_paired{1} );
	[ rs_r , rs_c ] = size( reads_paired{2} );
	[ r_r , r_c ] = size( reads_single );
	index_rf = [0:1:rf_r-1] * rf_c ;
	index_rs = [0:1:rs_r-1] * rs_c ;
	index_r = [0:1:r_r-1] * r_c ;
	if !isempty(single_nr_exoncoords)
		reads_single_startsite(:,1) = reads_single(:,1);	%Absolute startsite of the single end read on the genome
		reads_single_endsite(:,1) = (reads_single' ( single_nr_exoncoords' .+ index_r ))';	%Absolute endsite of the single end read on the genome
	end
	if ( !isempty(paired_nr_exoncoords{1}) && !isempty(paired_nr_exoncoords{2}) )
		reads_paired_first_startsite(:,1) = reads_paired{1}(:,1);	%Absolute startsite of the first read of a readpair on the genome
		reads_paired_first_endsite(:,1) = ( reads_paired{1}' ( paired_nr_exoncoords{1}' .+ index_rf ))' ;	%Absolute endsite of the first read of a readpair  on the genome
		reads_paired_second_startsite(:,1) = reads_paired{2}(:,1);	%Absolute startsite of the second read of a readpair  on the genome
		reads_paired_second_endsite(:,1) = (reads_paired{2}' ( paired_nr_exoncoords{2}' .+ index_rs ))' ;	%Absolute endsite of the second read of a readpair  on the genome
	end
	for iat=1:nr_transcripts
		% The for loop iterates through all " transcript variants ". Each variant constitutes different exclusion criteria for reads. This criteria is uniform to all reads though, hence OCTAVE vectorization enables a significant speedup compared to iterating through the reads. 
		trans_defs_line = gene{end}{iat}{end};
		gaps_line = (trans_defs_line(1,3:2:end) .- trans_defs_line(1,2:2:end-1)) -1;
		%%% Fragment lengths & startsites out of the reads_paired cell array
		if ( !isempty(paired_nr_exoncoords{1}) && !isempty(paired_nr_exoncoords{2}) )
			valid_paired_first = zeros(1,nr_reads_paired);
			valid_paired_second = zeros(1,nr_reads_paired);
			trans_comp_paired = repmat(trans_defs_line, nr_reads_paired , 1 );
			% *_startsite_transcript_exon_count variables are counting variables to track the number of exons that have positions with lower values than the read. this information is a simplified position index that is used to remove intron distances during the conversion to transcript space.
			reads_paired_first_startsite_transcript_exon_count = sum( repmat( reads_paired_first_startsite , 1 , nr_exons(iat)  ) >= trans_comp_paired(: , 1:2:end) , 2 )' ;
			reads_paired_first_endsite_transcript_exon_count = sum( repmat( reads_paired_first_endsite , 1 , nr_exons(iat)  ) >= trans_comp_paired(: , 1:2:end) , 2 )' ;
			reads_paired_second_startsite_transcript_exon_count = sum( repmat( reads_paired_second_startsite , 1 , nr_exons(iat)  ) >= trans_comp_paired(: , 1:2:end) , 2 )' ;
			reads_paired_second_endsite_transcript_exon_count = sum( repmat( reads_paired_second_endsite , 1 , nr_exons(iat)  ) >= trans_comp_paired(: , 1:2:end) , 2 )' ;
			maxrange=max( [paired_nr_exoncoords{1} ; paired_nr_exoncoords{2}] );
			for exonrange = 2:2:maxrange
				pattern=zeros(1,exonrange);
				pattern(1)=1;
				pattern(end)=-1;
				%All reads with 2..4...6 ... nr_exons(iat) exons
				%...Excluding the ones that would exceed the transcript definition
				index_first = paired_nr_exoncoords{1}(:)==exonrange;
				index_first(( (reads_paired_first_startsite_transcript_exon_count-1)*2 + exonrange ) > nr_exons(iat)*2 ) = 0;
				index_first(reads_paired_first_startsite_transcript_exon_count <= 0) = 0;
				index_second = paired_nr_exoncoords{2}(:)==exonrange;
				index_second(( (reads_paired_second_startsite_transcript_exon_count-1)*2 + exonrange) > nr_exons(iat)*2) = 0;
				index_second(reads_paired_second_startsite_transcript_exon_count <= 0) = 0;
				base_first=(repmat( [1:exonrange],nr_reads_paired,1) .+ repmat( 2*(reads_paired_first_startsite_transcript_exon_count-1)' , 1 , exonrange ));
				base_second=(repmat( [1:exonrange],nr_reads_paired,1) .+ repmat( 2*(reads_paired_second_startsite_transcript_exon_count-1)' , 1 , exonrange ));
				if any(index_first)
					span_first=sum(index_first);
					span_first_pattern = repmat(pattern,span_first,1);
					focus_reads = reads_paired{1}(index_first,1:exonrange);
					focus_trans = 	trans_defs_line(base_first(index_first,:));
					validity_mat_focus = sign(focus_reads .- focus_trans);
					validity_mat_focus ( validity_mat_focus (:,1) == 0 , 1) = 1;
					validity_mat_focus ( validity_mat_focus (:,exonrange) == 0 , exonrange) = -1;
					valid_paired_first(index_first) = all( validity_mat_focus == span_first_pattern , 2);
				end
				if any(index_second)
					span_second=sum(index_second);
					span_second_pattern = repmat(pattern,span_second,1);
					focus_reads = reads_paired{2}(index_second,1:exonrange);
					focus_trans = 	trans_defs_line(base_second(index_second,:));
					validity_mat_focus = sign(focus_reads .- focus_trans);
					validity_mat_focus ( validity_mat_focus (:,1) == 0 , 1) = 1;
					validity_mat_focus ( validity_mat_focus (:,exonrange) == 0 , exonrange ) = -1;
					valid_paired_second(index_second) = all( validity_mat_focus == span_second_pattern , 2);
				end
			end
			fragment_lengths_validity_mat_paired( 1:nr_reads_paired , iat ) = all ( [valid_paired_first ; valid_paired_second ] , 1 )';
			fragment_startsites_validity_mat_paired( 1:nr_reads_paired , iat ) = fragment_lengths_validity_mat_paired( 1:nr_reads_paired , iat );
			gaps = zeros(1,length(gaps_line)+1);
			gaps(2:end) = gaps_line;
			gaps_matrix = repmat(gaps,size(gaps,2),1);
			gaps_matrix = triu ( gaps_matrix , 1);
			for ii=1:1:size(gaps_matrix,2)
				gaps_matrix(:,size(gaps_matrix,2)+1-ii) = sum ( gaps_matrix(:,1:size(gaps_matrix,2)+1-ii),2) ;
			end
			reads_union_index = ( reads_paired_first_startsite_transcript_exon_count > 0 ) & ( reads_paired_second_endsite_transcript_exon_count > 0 ) & ( reads_paired_first_startsite_transcript_exon_count <= nr_exons(iat) ) & (reads_paired_second_endsite_transcript_exon_count <= nr_exons(iat) ) ;
			%Fragment lengths from paired end
			fragment_lengths_trans_coord_paired( reads_union_index , iat ) = (reads_paired_second_endsite(reads_union_index,1)' .- reads_paired_first_startsite(reads_union_index,1)' .- gaps_matrix( reads_paired_first_startsite_transcript_exon_count(reads_union_index) .+ (reads_paired_second_endsite_transcript_exon_count(reads_union_index) - 1 ) * size(gaps_matrix)(2)  ))' + 1;
			%Fragment starts from paired end
			fragment_startsites_trans_coord_paired( reads_union_index , iat ) = ( (reads_paired_first_startsite(reads_union_index,1)' - gene{end}{iat}{end}(1) ) .- gaps_matrix( reads_union_index(reads_union_index==1)  .+ (reads_paired_first_startsite_transcript_exon_count(reads_union_index) - 1 ) * size(gaps_matrix)(2)  ) + 1 )' ;
			if strcmp( gene{end}{iat}{2} , "-" )
				fragment_startsites_trans_coord_paired( reads_union_index , iat ) = repmat( trans_lengths(1,iat) , sum(reads_union_index) , 1 ) .- ( fragment_startsites_trans_coord_paired( reads_union_index , iat ) .+ fragment_lengths_trans_coord_paired(reads_union_index,iat) - 1 ) + 1  ;
			end
		end
		%%% Fragment startsites out of the reads_single matrix
		if !isempty(single_nr_exoncoords)
			valid_single = zeros(1,nr_reads_single);
			trans_comp_single = repmat(trans_defs_line, nr_reads_single , 1 );
			reads_single_startsite_transcript_exon_count = sum( repmat( reads_single_startsite , 1 , nr_exons(iat)  ) >= trans_comp_single(: , 1:2:end) , 2 )' ;
			reads_single_endsite_transcript_exon_count = sum( repmat( reads_single_endsite , 1 , nr_exons(iat)  ) >= trans_comp_single(: , 1:2:end) , 2 )' ;
			maxrange=max( single_nr_exoncoords );
			for exonrange = 2:2:maxrange
				pattern=zeros(1,exonrange); pattern(1)=1; pattern(end)=-1;
				%All reads with 2..4...6 ... nr_exons(iat) exons
				%...Excluding the ones that would exceed the transcript definition
				index_single = single_nr_exoncoords(:)==exonrange;
				index_single(( (reads_single_startsite_transcript_exon_count-1)*2 + exonrange ) > nr_exons(iat)*2 ) = 0;
				index_single(reads_single_startsite_transcript_exon_count <= 0) = 0;
				base=(repmat( [1:exonrange],nr_reads_single,1) .+ repmat( 2*(reads_single_startsite_transcript_exon_count-1)' , 1 , exonrange ));
				if any(index_single)
					span=sum(index_single);
					span_pattern = repmat(pattern,span,1);
					focus_reads = reads_single(index_single,1:exonrange);
					focus_trans = 	trans_defs_line(base(index_single,:));
					validity_mat_focus = sign(focus_reads .- focus_trans);
					validity_mat_focus ( validity_mat_focus (:,1) == 0 , 1) = 1;
					validity_mat_focus ( validity_mat_focus (:,exonrange) == 0 , exonrange) = -1;
					valid_single(index_single) = all( validity_mat_focus == span_pattern , 2);
				end
			end
			fragment_startsites_validity_mat_single( 1:nr_reads_single , iat ) = all ( valid_single , 1 )';
			fragment_lengths_validity_mat_single( 1:nr_reads_single , iat ) = zeros(nr_reads_single,1); # fragment_startsites_validity_mat_single( 1:nr_reads_single , iat ) * 0;
			gaps = zeros(1,length(gaps_line)+1);
			gaps(2:end) = gaps_line;
			gaps_matrix = repmat(gaps,size(gaps,2),1);
			gaps_matrix = triu ( gaps_matrix , 1);
			for ii=1:1:size(gaps_matrix,2)
				gaps_matrix(:,size(gaps_matrix,2)+1-ii) = sum ( gaps_matrix(:,1:size(gaps_matrix,2)+1-ii),2) ;
			end
			reads_union_index = ( reads_single_startsite_transcript_exon_count > 0 ) && ( reads_single_startsite_transcript_exon_count <= nr_exons(iat) ) ;
			%Fragment starts from single end
			fragment_startsites_trans_coord_single( reads_union_index , iat ) = ( (reads_single_startsite(reads_union_index,1)' - gene{end}{iat}{end}(1) ) .- gaps_matrix( reads_union_index(reads_union_index==1)  .+ (reads_single_startsite_transcript_exon_count(reads_union_index) - 1 ) * size(gaps_matrix)(2)  ) + 1 )' ;
			if strcmp( gene{end}{iat}{2} , "-" )
				%Read lengths from single end - for the purpose of swapping the direction in case of antisense reads.
				read_lengths_single_trans_coord( reads_union_index , iat ) = (reads_single_endsite(reads_union_index,1)' .- reads_single_startsite(reads_union_index,1)' .- gaps_matrix( reads_single_startsite_transcript_exon_count(reads_union_index) .+ (reads_single_endsite_transcript_exon_count(reads_union_index) - 1 ) * size(gaps_matrix)(2)  ))' ;
				fragment_startsites_trans_coord( reads_union_index , iat ) = repmat( trans_lengths(1,iat) , sum(reads_union_index) , 1 ) .- ( fragment_startsites_trans_coord_single( reads_union_index , iat ) .+ read_lengths_single_trans_coord( reads_union_index , iat ) - 1 ) + 1 ;
			end
			fragment_lengths_trans_coord_single(:,iat) = zeros(nr_reads_single,1); # fragment_startsites_trans_coord_single(:,iat) * 0;
		end
	end
	%OUT_OF_BOUNDS
	out_of_bounds_index_paired=sum(fragment_startsites_validity_mat_paired,2) ==0 ;
	out_of_bounds_index_single=sum(fragment_startsites_validity_mat_single,2) ==0 ;
	fragment_startsites_validity_mat_paired(out_of_bounds_index_paired,:)=[];
	fragment_startsites_validity_mat_single(out_of_bounds_index_single,:)=[];
	fragment_lengths_validity_mat_paired(out_of_bounds_index_paired,:)=[];
	fragment_lengths_validity_mat_single(out_of_bounds_index_single,:)=[];
	fragment_startsites_trans_coord_paired(out_of_bounds_index_paired,:)=[];
	fragment_startsites_trans_coord_single(out_of_bounds_index_single,:)=[];
	fragment_lengths_trans_coord_paired(out_of_bounds_index_paired,:)=[];
	fragment_lengths_trans_coord_single(out_of_bounds_index_single,:)=[];
	nr_reads_locus = size ( fragment_startsites_validity_mat_paired,1 ) * 2 + size ( fragment_startsites_validity_mat_single , 1 );
	fragment_startsites_validity_mat = [ fragment_startsites_validity_mat_paired ; fragment_startsites_validity_mat_single ];
	fragment_lengths_validity_mat = [ fragment_lengths_validity_mat_paired ; fragment_lengths_validity_mat_single ];
	fragment_startsites_trans_coord = [ fragment_startsites_trans_coord_paired ; fragment_startsites_trans_coord_single ];
	fragment_lengths_trans_coord = [ fragment_lengths_trans_coord_paired ; fragment_lengths_trans_coord_single ];
	nr_frag_starts = size( fragment_startsites_trans_coord , 1 );
	fragment_startsites_trans_coord = fragment_startsites_trans_coord .* fragment_startsites_validity_mat;
	fragment_startsites_trans_coord(fragment_startsites_trans_coord==0) = 1;
	fragment_lengths_trans_coord = fragment_lengths_trans_coord .* fragment_lengths_validity_mat;
	fragment_lengths_trans_coord(fragment_lengths_trans_coord==0) = 1;
	frags_out_paired=fragment_ids_paired(out_of_bounds_index_paired);
	frags_out_single=fragment_ids_single(out_of_bounds_index_single);
	if any([out_of_bounds_index_paired;out_of_bounds_index_single])
		sum_out_of_bounds_fragments=sum([out_of_bounds_index_paired;out_of_bounds_index_single])
		if( sum ( sum_out_of_bounds_fragments ) == totalcount )
			error("all fragments were deemed invalid for all transcripts! \n")
		end
	end
endfunction
